"use strict";
var __makeTemplateObject = (this && this.__makeTemplateObject) || function (cooked, raw) {
    if (Object.defineProperty) { Object.defineProperty(cooked, "raw", { value: raw }); } else { cooked.raw = raw; }
    return cooked;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var LargeObject_1 = __importDefault(require("./LargeObject"));
var util_1 = require("util");
var MODE;
(function (MODE) {
    MODE[MODE["WRITE"] = 131072] = "WRITE";
    MODE[MODE["READ"] = 262144] = "READ";
    MODE[MODE["READWRITE"] = 393216] = "READWRITE";
})(MODE || (MODE = {}));
/** This class lets you use the Large Object functionality of PostgreSQL.
 * All usage of Large Object should take place within a transaction block!
 * (BEGIN ... COMMIT)
 *
 * @example new LargeObjectManager(client)
 * @constructor
 * @exports postgres-large-object/lib/LargeObjectManager
 * @param {module:postgres/Sql} sql A postgres (https://www.npmjs.com/package/postgres) transaction
 *         object as given by `sql.begin(fn)`
 */
var LargeObjectManager = /** @class */ (function () {
    function LargeObjectManager(sql) {
        var _this = this;
        /** Open an existing large object, based on its OID.
         * In mode READ, the data read from it will reflect the
         * contents of the large object at the time of the transaction
         * snapshot that was active when open was executed,
         * regardless of later writes by this or other transactions.
         * If opened using WRITE (or READWRITE), data read will reflect
         * all writes of other committed transactions as well as
         * writes of the current transaction.
         * @param {Number} oid
         * @param {Number} mode One of WRITE, READ, or READWRITE
         * @returns {Promise.<module:postgres-large-object/lib/LargeObject>}
         */
        this.openAsync = function (oid, mode) { return __awaiter(_this, void 0, void 0, function () {
            var fd;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (!oid) {
                            throw Error('Illegal Argument');
                        }
                        return [4 /*yield*/, this._sql(templateObject_1 || (templateObject_1 = __makeTemplateObject(["SELECT lo_open(", ", ", ") AS fd"], ["SELECT lo_open(", ", ", ") AS fd"])), oid, mode)];
                    case 1:
                        fd = (_a.sent())[0].fd;
                        return [2 /*return*/, new LargeObject_1["default"](this._sql, oid, fd)];
                }
            });
        }); };
        /** @callback module:postgres-large-object/lib/LargeObjectManager~openCallback
         * @param {?Error} error If set, an error occurred.
         * @param {module:postgres-large-object/lib/LargeObject} result
         */
        /** Open an existing large object, based on its OID.
         * In mode READ, the data read from it will reflect the
         * contents of the large object at the time of the transaction
         * snapshot that was active when open was executed,
         * regardless of later writes by this or other transactions.
         * If opened using WRITE (or READWRITE), data read will reflect
         * all writes of other committed transactions as well as
         * writes of the current transaction.
         * @param {Number} oid
         * @param {Number} mode One of WRITE, READ, or READWRITE
         * @param {module:postgres-large-object/lib/LargeObjectManager~openCallback} callback
         */
        this.open = util_1.callbackify(this.openAsync);
        /** Creates a large object, returning its OID.
         * After which you can open() it.
         * @returns {Promise.<number>} oid
         */
        this.createAsync = function () { return __awaiter(_this, void 0, void 0, function () {
            var oid;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this._sql(templateObject_2 || (templateObject_2 = __makeTemplateObject(["SELECT lo_creat(", ") as oid"], ["SELECT lo_creat(", ") as oid"])), LargeObjectManager.READWRITE)];
                    case 1:
                        oid = (_a.sent())[0].oid;
                        return [2 /*return*/, oid];
                }
            });
        }); };
        /** @callback module:postgres-large-object/lib/LargeObjectManager~createCallback
         * @param {?Error} error If set, an error occurred.
         * @param {Number} oid
         */
        /** Creates a large object, returning its OID.
         * After which you can open() it.
         * @param {module:postgres-large-object/lib/LargeObjectManager~createCallback} callback
         */
        this.create = util_1.callbackify(this.createAsync);
        /** Unlinks (deletes) a large object
         * @param {number} oid
         * @returns {Promise}
         */
        this.unlinkAsync = function (oid) { return __awaiter(_this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (!oid) {
                            throw Error('Illegal Argument');
                        }
                        return [4 /*yield*/, this._sql(templateObject_3 || (templateObject_3 = __makeTemplateObject(["SELECT lo_unlink(", ") AS ok"], ["SELECT lo_unlink(", ") AS ok"])), oid)];
                    case 1:
                        _a.sent();
                        return [2 /*return*/];
                }
            });
        }); };
        /** @callback module:postgres-large-object/lib/LargeObjectManager~unlinkCallback
         * @param {?Error} error If set, an error occurred.
         */
        /** Unlinks (deletes) a large object
         * @param {number} oid
         * @param {module:postgres-large-object/lib/LargeObjectManager~unlinkCallback} [callback]
         */
        this.unlink = util_1.callbackify(this.unlinkAsync);
        /** Open a large object, return a stream and close the object when done streaming.
         * Only call this within a transaction block.
         * @param {Number} oid
         * @param {Number} [bufferSize=16384]
         * @returns {Promise.<Array>} The total size and a ReadStream
         *
         */
        this.openAndReadableStreamAsync = function (oid, bufferSize) {
            if (bufferSize === void 0) { bufferSize = 16384; }
            return __awaiter(_this, void 0, void 0, function () {
                var obj, size, stream;
                var _this = this;
                return __generator(this, function (_a) {
                    switch (_a.label) {
                        case 0: return [4 /*yield*/, this.openAsync(oid, LargeObjectManager.READ)];
                        case 1:
                            obj = _a.sent();
                            return [4 /*yield*/, obj.sizeAsync()];
                        case 2:
                            size = _a.sent();
                            stream = obj.getReadableStream(bufferSize);
                            stream.on('end', function () { return __awaiter(_this, void 0, void 0, function () {
                                var err_1;
                                return __generator(this, function (_a) {
                                    switch (_a.label) {
                                        case 0:
                                            _a.trys.push([0, 2, , 3]);
                                            return [4 /*yield*/, obj.closeAsync()];
                                        case 1:
                                            _a.sent();
                                            return [3 /*break*/, 3];
                                        case 2:
                                            err_1 = _a.sent();
                                            console.error('Warning: closing a large object failed:', err_1);
                                            return [3 /*break*/, 3];
                                        case 3: return [2 /*return*/];
                                    }
                                });
                            }); });
                            return [2 /*return*/, [size, stream]];
                    }
                });
            });
        };
        /** @callback module:postgres-large-object/lib/LargeObjectManager~openAndReadableStreamCallback
         * @param {?Error} error If set, an error occurred.
         * @param {Number} size The total size of the large object
         * @param {module:postgres-large-object/lib/ReadStream} stream
         */
        /** Open a large object, return a stream and close the object when done streaming.
         * Only call this within a transaction block.
         * @param {Number} oid
         * @param {Number} [bufferSize=16384]
         * @param {module:postgres-large-object/lib/LargeObjectManager~openAndReadableStreamCallback} callback
         *
         */
        this.openAndReadableStream = function (oid) {
            var args = [];
            for (var _i = 1; _i < arguments.length; _i++) {
                args[_i - 1] = arguments[_i];
            }
            var bufferSize = util_1.isNumber(args[0]) ? args[0] : 16384;
            var callback = (util_1.isNumber(args[0]) ? args[1] : args[0]);
            _this.openAndReadableStreamAsync(oid, bufferSize)
                .then(function (_a) {
                var size = _a[0], stream = _a[1];
                return callback(undefined, size, stream);
            })["catch"](function (err) { return callback(err); });
        };
        /** Create and open a large object, return a stream and close the object when done streaming.
         * Only call this within a transaction block.
         * @param {Number} [bufferSize=16384]
         * @returns {promise.<Array>} The oid and a WriteStream
         */
        this.createAndWritableStreamAsync = function (bufferSize) {
            if (bufferSize === void 0) { bufferSize = 16384; }
            return __awaiter(_this, void 0, void 0, function () {
                var oid, obj, stream;
                var _this = this;
                return __generator(this, function (_a) {
                    switch (_a.label) {
                        case 0: return [4 /*yield*/, this.createAsync()];
                        case 1:
                            oid = _a.sent();
                            return [4 /*yield*/, this.openAsync(oid, LargeObjectManager.WRITE)];
                        case 2:
                            obj = _a.sent();
                            stream = obj.getWritableStream(bufferSize);
                            stream.on('finish', function () { return __awaiter(_this, void 0, void 0, function () {
                                var err_2;
                                return __generator(this, function (_a) {
                                    switch (_a.label) {
                                        case 0:
                                            _a.trys.push([0, 2, , 3]);
                                            return [4 /*yield*/, obj.closeAsync()];
                                        case 1:
                                            _a.sent();
                                            return [3 /*break*/, 3];
                                        case 2:
                                            err_2 = _a.sent();
                                            console.error('Warning: closing a large object failed:', err_2);
                                            return [3 /*break*/, 3];
                                        case 3: return [2 /*return*/];
                                    }
                                });
                            }); });
                            return [2 /*return*/, [oid, stream]];
                    }
                });
            });
        };
        /** @callback module:postgres-large-object/lib/LargeObjectManager~createAndWritableStreamCallback
         * @param {?Error} error If set, an error occurred.
         * @param {Number} oid
         * @param {module:postgres-large-object/lib/WriteStream} stream
         */
        /** Create and open a large object, return a stream and close the object when done streaming.
         * Only call this within a transaction block.
         * @param {Number} [bufferSize=16384]
         * @param {module:postgres-large-object/lib/LargeObjectManager~createAndWritableStreamCallback} [callback]
         */
        this.createAndWritableStream = function () {
            var args = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                args[_i] = arguments[_i];
            }
            var bufferSize = util_1.isNumber(args[0]) ? args[0] : 16384;
            var callback = (util_1.isNumber(args[0]) ? args[1] : args[0]);
            _this.createAndWritableStreamAsync(bufferSize)
                .then(function (_a) {
                var oid = _a[0], stream = _a[1];
                return callback(undefined, oid, stream);
            })["catch"](function (err) { return callback(err); });
        };
        this._sql = sql;
    }
    LargeObjectManager.WRITE = MODE.WRITE;
    LargeObjectManager.READ = MODE.READ;
    LargeObjectManager.READWRITE = MODE.READWRITE;
    return LargeObjectManager;
}());
var templateObject_1, templateObject_2, templateObject_3;
module.exports = LargeObjectManager;
